#ifndef PYTHONIC_INCLUDE_NUMPY_ROUND__HPP
#define PYTHONIC_INCLUDE_NUMPY_ROUND__HPP

#include "pythonic/include/numpy/around.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{
  USING_FUNCTOR(round_, around);
}
PYTHONIC_NS_END

#endif
