#ifndef PYTHONIC_IO__IO_TEXTIOWRAPPER_FILENO_HPP
#define PYTHONIC_IO__IO_TEXTIOWRAPPER_FILENO_HPP

#include "pythonic/include/io/_io/TextIOWrapper/fileno.hpp"
#include "pythonic/builtins/file/fileno.hpp"
#endif
