#ifndef PYTHONIC_INCLUDE_STRING_ASCII_UPPERCASE_HPP
#define PYTHONIC_INCLUDE_STRING_ASCII_UPPERCASE_HPP

#include "pythonic/types/str.hpp"

PYTHONIC_NS_BEGIN

namespace string
{
  static types::str const ascii_uppercase("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
}
PYTHONIC_NS_END

#endif
