#ifndef PYTHONIC_NUMPY_ADD_REDUCE_HPP
#define PYTHONIC_NUMPY_ADD_REDUCE_HPP

#define UFUNC_NAME add
#define UFUNC_INAME iadd
#include "pythonic/include/numpy/add/reduce.hpp"
#include "pythonic/numpy/ufunc_reduce.hpp"
#undef UFUNC_NAME
#undef UFUNC_INAME

#endif
