#ifndef PYTHONIC_INCLUDE_OPERATOR_NOT__HPP
#define PYTHONIC_INCLUDE_OPERATOR_NOT__HPP

#include "pythonic/include/operator_/not_.hpp"

PYTHONIC_NS_BEGIN

namespace operator_
{
  USING_FUNCTOR(__not__, not_);
}
PYTHONIC_NS_END

#endif
