#ifndef PYTHONIC_OPERATOR_IOR_HPP
#define PYTHONIC_OPERATOR_IOR_HPP

#include "pythonic/include/operator_/ior.hpp"

#define OPERATOR_NAME ior
#define OPERATOR_SYMBOL |
#define OPERATOR_ISYMBOL |=

#include "pythonic/operator_/icommon.hpp"

#endif
