#ifndef PYTHONIC_INCLUDE_BUILTIN_NONE_HPP
#define PYTHONIC_INCLUDE_BUILTIN_NONE_HPP

#include "pythonic/types/NoneType.hpp"

PYTHONIC_NS_BEGIN

namespace builtins
{

  types::none_type const None;
}
PYTHONIC_NS_END

#endif
