#ifndef PYTHONIC_INCLUDE_OPERATOR_IAND_HPP
#define PYTHONIC_INCLUDE_OPERATOR_IAND_HPP

#define OPERATOR_NAME iand
#define OPERATOR_SYMBOL &
#define OPERATOR_ISYMBOL &=
#include "pythonic/include/operator_/icommon.hpp"

#endif
