#ifndef PYTHONIC_INCLUDE_MATH_PI_HPP
#define PYTHONIC_INCLUDE_MATH_PI_HPP

PYTHONIC_NS_BEGIN

namespace math
{
  // see https://meetingcpp.com/blog/items/cpp-and-pi.html
  double constexpr pi =
      3.14159265358979323846264338327950288419716939937510582097494459230781640628620899862803482534211706798214808651e+00;
}
PYTHONIC_NS_END

#endif
